@@ -30,11 +30,16 @@ from __future__ import unicode_literals
 
 import unittest
 import datetime
+import sys
+import os
 
 import tuf
 import tuf.formats
 
+from tests import utils
+
 import securesystemslib
+import securesystemslib.util
 import six
 
 
@@ -256,6 +261,12 @@ class TestFormats(unittest.TestCase):
          'confined_target_dirs': ['path1/', 'path2/'],
          'custom': {'type': 'mirror'}}),
 
+      'MIRROR_SCHEMA_NO_CONFINED_TARGETS': (tuf.formats.MIRROR_SCHEMA,
+        {'url_prefix': 'http://localhost:8001',
+         'metadata_path': 'metadata/',
+         'targets_path': 'targets/',
+         'custom': {'type': 'mirror'}}),
+
       'MIRRORDICT_SCHEMA': (tuf.formats.MIRRORDICT_SCHEMA,
         {'mirror1': {'url_prefix': 'http://localhost:8001',
          'metadata_path': 'metadata/',
@@ -718,6 +729,23 @@ class TestFormats(unittest.TestCase):
 
 
 
+  def test_expiry_string_to_datetime(self):
+    dt = tuf.formats.expiry_string_to_datetime('1985-10-21T13:20:00Z')
+    self.assertEqual(dt, datetime.datetime(1985, 10, 21, 13, 20, 0))
+    dt = tuf.formats.expiry_string_to_datetime('2038-01-19T03:14:08Z')
+    self.assertEqual(dt, datetime.datetime(2038, 1, 19, 3, 14, 8))
+
+    # First 3 fail via securesystemslib schema, last one because of strptime()
+    invalid_inputs = [
+      '2038-1-19T03:14:08Z', # leading zeros not optional
+      '2038-01-19T031408Z', # strict time parsing
+      '2038-01-19T03:14:08Z-06:00', # timezone not allowed
+      '2038-13-19T03:14:08Z', # too many months
+    ]
+    for invalid_input in invalid_inputs:
+      with self.assertRaises(securesystemslib.exceptions.FormatError):
+        tuf.formats.expiry_string_to_datetime(invalid_input)
+
 
 
   def test_unix_timestamp_to_datetime(self):
@@ -775,20 +803,10 @@ class TestFormats(unittest.TestCase):
 
   def test_make_signable(self):
     # Test conditions for expected make_signable() behavior.
-    root = {'_type': 'root',
-            'spec_version': '1.0.0',
-            'version': 8,
-            'consistent_snapshot': False,
-            'expires': '1985-10-21T13:20:00Z',
-            'keys': {'123abc': {'keytype': 'rsa',
-                                'scheme': 'rsassa-pss-sha256',
-                                'keyval': {'public': 'pubkey',
-                                           'private': 'privkey'}}},
-            'roles': {'root': {'keyids': ['123abc'],
-                               'threshold': 1,
-                               'paths': ['path1/', 'path2']}}}
-
     SIGNABLE_SCHEMA = tuf.formats.SIGNABLE_SCHEMA
+    root_file = os.path.join('repository_data', 'repository', 'metadata',
+        'root.json')
+    root = securesystemslib.util.load_json_file(root_file)
     self.assertTrue(SIGNABLE_SCHEMA.matches(tuf.formats.make_signable(root)))
     signable = tuf.formats.make_signable(root)
     self.assertEqual('root', tuf.formats.check_signable_object_format(signable))
@@ -899,19 +917,9 @@ class TestFormats(unittest.TestCase):
 
   def test_check_signable_object_format(self):
     # Test condition for a valid argument.
-    root = {'_type': 'root',
-            'spec_version': '1.0.0',
-            'version': 8,
-            'consistent_snapshot': False,
-            'expires': '1985-10-21T13:20:00Z',
-            'keys': {'123abc': {'keytype': 'rsa',
-                                'scheme': 'rsassa-pss-sha256',
-                                'keyval': {'public': 'pubkey',
-                                           'private': 'privkey'}}},
-            'roles': {'root': {'keyids': ['123abc'],
-                               'threshold': 1,
-                               'paths': ['path1/', 'path2']}}}
-
+    root_file = os.path.join('repository_data', 'repository', 'metadata',
+        'root.json')
+    root = securesystemslib.util.load_json_file(root_file)
     root = tuf.formats.make_signable(root)
     self.assertEqual('root', tuf.formats.check_signable_object_format(root))
 
@@ -968,4 +976,5 @@ class TestFormats(unittest.TestCase):
 
 # Run unit test.
 if __name__ == '__main__':
+  utils.configure_test_logging(sys.argv)
   unittest.main()
