@@ -35,10 +35,9 @@ import logging
 import tempfile
 import json
 import shutil
-import stat
-import sys
 import unittest
-import platform
+import copy
+import sys
 
 import tuf
 import tuf.formats
@@ -51,6 +50,8 @@ import tuf.settings
 import tuf.repository_lib as repo_lib
 import tuf.repository_tool as repo_tool
 
+from tests import utils
+
 import securesystemslib
 import securesystemslib.exceptions
 import securesystemslib.rsa_keys
@@ -69,9 +70,6 @@ TOP_LEVEL_METADATA_FILES = ['root.json', 'targets.json', 'timestamp.json',
 class TestRepositoryToolFunctions(unittest.TestCase):
   @classmethod
   def setUpClass(cls):
-
-    # setUpClass() is called before tests in an individual class are executed.
-
     # Create a temporary directory to store the repository, metadata, and target
     # files.  'temporary_directory' must be deleted in TearDownClass() so that
     # temporary files are always removed, even when exceptions occur.
@@ -83,10 +81,6 @@ class TestRepositoryToolFunctions(unittest.TestCase):
 
   @classmethod
   def tearDownClass(cls):
-
-    # tearDownModule() is called after all the tests have run.
-    # http://docs.python.org/2/library/unittest.html#class-and-module-fixtures
-
     # Remove the temporary repository directory, which should contain all the
     # metadata, targets, and key files generated for the test cases.
     tuf.roledb.clear_roledb(clear_all=True)
@@ -149,7 +143,7 @@ class TestRepositoryToolFunctions(unittest.TestCase):
     temporary_directory = tempfile.mkdtemp(dir=self.temporary_directory)
     ed25519_keypath = os.path.join(temporary_directory, 'ed25519_key')
     securesystemslib.interface.generate_and_write_ed25519_keypair(
-        ed25519_keypath, password='pw')
+        password='pw', filepath=ed25519_keypath)
 
     imported_ed25519_key = \
       repo_lib.import_ed25519_privatekey_from_file(ed25519_keypath, 'pw')
@@ -313,11 +307,6 @@ class TestRepositoryToolFunctions(unittest.TestCase):
     # securesystemslib.exceptions.Error exception is raised for duplicate keyids.
     tuf.keydb._keydb_dict['default'][root_keyids[0]]['keytype'] = 'rsa'
 
-    # Add duplicate keyid to root's roleinfo.
-    tuf.roledb._roledb_dict['default']['root']['keyids'].append(root_keyids[0])
-    self.assertRaises(securesystemslib.exceptions.Error, repo_lib.generate_root_metadata, 1,
-                      expires, consistent_snapshot=False)
-
     # Test improperly formatted arguments.
     self.assertRaises(securesystemslib.exceptions.FormatError, repo_lib.generate_root_metadata,
                       '3', expires, False)
@@ -352,48 +341,55 @@ class TestRepositoryToolFunctions(unittest.TestCase):
     file_permissions = oct(os.stat(file1_path).st_mode)[4:]
     target_files = {'file.txt': {'custom': {'file_permission': file_permissions}}}
 
-    delegations = {"keys": {
-      "a394c28384648328b16731f81440d72243c77bb44c07c040be99347f0df7d7bf": {
-       "keytype": "ed25519",
-       "keyval": {
-        "public": "3eb81026ded5af2c61fb3d4b272ac53cd1049a810ee88f4df1fc35cdaf918157"
-       }
-      }
-     },
-     "roles": [
-      {
-       "keyids": [
-        "a394c28384648328b16731f81440d72243c77bb44c07c040be99347f0df7d7bf"
-       ],
-       "name": "targets/warehouse",
-       "paths": [
-        "/file1.txt", "/README.txt", '/warehouse/'
-       ],
-       "threshold": 1
-      }
-     ]
-    }
+    # Delegations data must be loaded into roledb since
+    # generate_targets_metadata tries to update delegations keyids
+    # and threshold
+    repository_path = os.path.join('repository_data', 'repository')
+    repository = repo_tool.load_repository(repository_path)
+    roleinfo = tuf.roledb.get_roleinfo('targets')
+    delegations = roleinfo['delegations']
 
-    targets_metadata = \
-      repo_lib.generate_targets_metadata(targets_directory, target_files,
-                                         version, expiration_date, delegations,
-                                         False)
+    targets_metadata = repo_lib.generate_targets_metadata(targets_directory,
+        target_files, version, expiration_date, delegations, False)
     self.assertTrue(tuf.formats.TARGETS_SCHEMA.matches(targets_metadata))
 
     # Valid arguments with 'delegations' set to None.
-    targets_metadata = \
-      repo_lib.generate_targets_metadata(targets_directory, target_files,
-                                         version, expiration_date, None,
-                                         False)
+    targets_metadata = repo_lib.generate_targets_metadata(targets_directory,
+        target_files, version, expiration_date, None, False)
     self.assertTrue(tuf.formats.TARGETS_SCHEMA.matches(targets_metadata))
 
+    # Test update in targets' delegations
+    keystore_path = os.path.join('repository_data', 'keystore')
+    targets_public_keypath = os.path.join(keystore_path, 'targets_key.pub')
+    targets_public_key = securesystemslib.interface.\
+        import_ed25519_publickey_from_file(targets_public_keypath)
+
+    # Add new key and threshold to delegated role
+    repository.targets('role1').add_verification_key(targets_public_key)
+    repository.targets('role1').threshold = 2
+    role1_keyids = tuf.roledb.get_role_keyids('role1')
+    role1_threshold = tuf.roledb.get_role_threshold('role1')
+    roleinfo = tuf.roledb.get_roleinfo('targets')
+    delegations = roleinfo['delegations']
+    old_delegations = copy.deepcopy(delegations)
+
+    targets_metadata  = repo_lib.generate_targets_metadata(targets_directory,
+        target_files, version, expiration_date, delegations, False)
+    self.assertNotEqual(old_delegations, delegations)
+    self.assertEqual(role1_keyids,
+        targets_metadata['delegations']['roles'][0]['keyids'])
+    self.assertEqual(role1_threshold,
+        targets_metadata['delegations']['roles'][0]['threshold'])
+    for keyid in role1_keyids:
+      self.assertIn(keyid, targets_metadata['delegations']['keys'])
+
+
     # Verify that 'digest.filename' file is saved to 'targets_directory' if
     # the 'write_consistent_targets' argument is True.
     list_targets_directory = os.listdir(targets_directory)
-    targets_metadata = \
-      repo_lib.generate_targets_metadata(targets_directory, target_files,
-                                          version, expiration_date, delegations,
-                                          write_consistent_targets=True)
+    targets_metadata = repo_lib.generate_targets_metadata(targets_directory,
+        target_files, version, expiration_date, delegations,
+        write_consistent_targets=True)
     new_list_targets_directory = os.listdir(targets_directory)
 
     # Verify that 'targets_directory' contains only one extra item.
@@ -437,6 +433,39 @@ class TestRepositoryToolFunctions(unittest.TestCase):
                       expiration_date)
 
 
+    # Test use of an existing fileinfo structures
+    target1_hashes = {'sha256': 'c2986576f5fdfd43944e2b19e775453b96748ec4fe2638a6d2f32f1310967095'}
+    target2_hashes = {'sha256': '517c0ce943e7274a2431fa5751e17cfd5225accd23e479bfaad13007751e87ef'}
+
+    # Test missing expected field, hashes, when use_existing_fileinfo
+    target_files = {'file.txt': {'length': 555}}
+    self.assertRaises(securesystemslib.exceptions.Error, repo_lib.generate_targets_metadata,
+                      targets_directory, target_files, version, expiration_date,
+                      use_existing_fileinfo=True)
+
+    # Test missing expected field, length, when use_existing_fileinfo
+    target_files = {'file.txt': {'hashes': target1_hashes}}
+    self.assertRaises(securesystemslib.exceptions.Error, repo_lib.generate_targets_metadata,
+                      targets_directory, target_files, version, expiration_date,
+                      use_existing_fileinfo=True)
+
+    # Test missing both expected fields when use_existing_fileinfo
+    target_files = {'file.txt': {}}
+    self.assertRaises(securesystemslib.exceptions.Error, repo_lib.generate_targets_metadata,
+                      targets_directory, target_files, version, expiration_date,
+                      use_existing_fileinfo=True)
+
+    target_files = {'file1.txt': {'custom': {'meta': 'foo'},
+                                  'hashes': target1_hashes,
+                                  'length': 555},
+                    'file2.txt': {'custom': {'meta': 'bar'},
+                                  'hashes': target2_hashes,
+                                  'length': 42}}
+    targets_metadata = \
+      repo_lib.generate_targets_metadata(targets_directory, target_files,
+                                         version, expiration_date, delegations,
+                                         False, use_existing_fileinfo=True)
+
 
   def _setup_generate_snapshot_metadata_test(self):
     # Test normal case.
@@ -856,6 +885,8 @@ class TestRepositoryToolFunctions(unittest.TestCase):
     # (specifically 'snapshot') and keys to be available in 'tuf.roledb'.
     tuf.roledb.create_roledb_from_root_metadata(root_signable['signed'],
         repository_name)
+    tuf.keydb.create_keydb_from_root_metadata(root_signable['signed'],
+        repository_name)
     temporary_directory = tempfile.mkdtemp(dir=self.temporary_directory)
     targets_directory = os.path.join(temporary_directory, 'targets')
     os.mkdir(targets_directory)
@@ -868,6 +899,14 @@ class TestRepositoryToolFunctions(unittest.TestCase):
     securesystemslib.util.ensure_parent_dir(obsolete_metadata)
     shutil.copyfile(targets_metadata, obsolete_metadata)
 
+    keystore_path = os.path.join('repository_data', 'keystore')
+    targets_private_keypath = os.path.join(keystore_path, 'targets_key')
+    targets_private_key = repo_lib.import_ed25519_privatekey_from_file(targets_private_keypath,
+        'password')
+    tuf.keydb.remove_key(targets_private_key['keyid'],
+        repository_name=repository_name)
+    tuf.keydb.add_key(targets_private_key, repository_name=repository_name)
+
     # Verify that obsolete metadata (a metadata file exists on disk, but the
     # role is unavailable in 'tuf.roledb').  First add the obsolete
     # role to 'tuf.roledb' so that its metadata file can be written to disk.
@@ -877,6 +916,7 @@ class TestRepositoryToolFunctions(unittest.TestCase):
       tuf.formats.unix_timestamp_to_datetime(int(time.time() + 86400))
     expiration = expiration.isoformat() + 'Z'
     targets_roleinfo['expires'] = expiration
+    targets_roleinfo['signing_keyids'] = targets_roleinfo['keyids']
     tuf.roledb.add_role('obsolete_role', targets_roleinfo,
         repository_name=repository_name)
 
@@ -961,13 +1001,38 @@ class TestRepositoryToolFunctions(unittest.TestCase):
     repository = repo_tool.create_new_repository(repository_directory, repository_name)
     repo_lib._load_top_level_metadata(repository, filenames, repository_name)
 
+    # Manually add targets delegations to roledb since
+    # repository.write('targets') will try to update its delegations
+    targets_filepath = os.path.join('repository_data', 'repository',
+                                 'metadata', 'targets.json')
+    targets_signable = securesystemslib.util.load_json_file(targets_filepath)
+    delegations = targets_signable['signed']['delegations']
+
+    roleinfo = {}
+    roleinfo['name'] = delegations['roles'][0]['name']
+    roleinfo['keyids'] = delegations['roles'][0]['keyids']
+    roleinfo['threshold'] = delegations['roles'][0]['threshold']
+    roleinfo['version'] = 1
+    tuf.roledb.add_role('role1', roleinfo, repository_name)
+
+    keystore_path = os.path.join('repository_data', 'keystore')
+    root_privkey_path = os.path.join(keystore_path, 'root_key')
+    targets_privkey_path = os.path.join(keystore_path, 'targets_key')
+    snapshot_privkey_path = os.path.join(keystore_path, 'snapshot_key')
+    timestamp_privkey_path = os.path.join(keystore_path, 'timestamp_key')
+
+    repository.root.load_signing_key(repo_lib.import_rsa_privatekey_from_file(root_privkey_path, 'password'))
+    repository.targets.load_signing_key(repo_lib.import_ed25519_privatekey_from_file(targets_privkey_path, 'password'))
+    repository.snapshot.load_signing_key(repo_lib.import_ed25519_privatekey_from_file(snapshot_privkey_path, 'password'))
+    repository.timestamp.load_signing_key(repo_lib.import_ed25519_privatekey_from_file(timestamp_privkey_path, 'password'))
+
     # Partially write all top-level roles (we increase the threshold of each
     # top-level role so that they are flagged as partially written.
     repository.root.threshold = repository.root.threshold + 1
     repository.snapshot.threshold = repository.snapshot.threshold + 1
     repository.targets.threshold = repository.targets.threshold + 1
     repository.timestamp.threshold = repository.timestamp.threshold + 1
-    repository.write('root', )
+    repository.write('root')
     repository.write('snapshot')
     repository.write('targets')
     repository.write('timestamp')
@@ -1042,4 +1107,5 @@ class TestRepositoryToolFunctions(unittest.TestCase):
 
 # Run the test cases.
 if __name__ == '__main__':
+  utils.configure_test_logging(sys.argv)
   unittest.main()
