@@ -36,20 +36,19 @@ import logging
 import tempfile
 import shutil
 import sys
-import errno
 
 import tuf
 import tuf.log
 import tuf.formats
 import tuf.roledb
 import tuf.keydb
-
 import tuf.repository_tool as repo_tool
-import securesystemslib.exceptions
+
+from tests import utils
 
 import securesystemslib
+import securesystemslib.exceptions
 import securesystemslib.storage
-import six
 
 logger = logging.getLogger(__name__)
 
@@ -59,9 +58,6 @@ repo_tool.disable_console_log_messages()
 class TestRepository(unittest.TestCase):
   @classmethod
   def setUpClass(cls):
-
-    # setUpClass() is called before tests in an individual class are executed.
-
     # Create a temporary directory to store the repository, metadata, and target
     # files.  'temporary_directory' must be deleted in TearDownClass() so that
     # temporary files are always removed, even when exceptions occur.
@@ -70,10 +66,6 @@ class TestRepository(unittest.TestCase):
 
   @classmethod
   def tearDownClass(cls):
-
-    # tearDownModule() is called after all the tests have run.
-    # http://docs.python.org/2/library/unittest.html#class-and-module-fixtures
-
     # Remove the temporary repository directory, which should contain all the
     # metadata, targets, and key files generated for the test cases.
     shutil.rmtree(cls.temporary_directory)
@@ -724,6 +716,34 @@ class TestRepository(unittest.TestCase):
       tuf.formats.check_signable_object_format(role_signable)
 
 
+  def test_signature_order(self):
+    """Test signatures are added to metadata in alphabetical order. """
+    # Create empty repo dir and init default repo in memory
+    repo_dir = tempfile.mkdtemp(dir=self.temporary_directory)
+    repo = repo_tool.create_new_repository(repo_dir)
+
+    # Dedicate any two existing test keys as root signing keys
+    for key_name in ["targets_key", "snapshot_key"]:
+      repo.root.load_signing_key(
+          repo_tool.import_ed25519_privatekey_from_file(
+              os.path.join("repository_data", "keystore", key_name),
+              "password"))
+
+    # Write root metadata with two signatures
+    repo.write("root")
+
+    # Load signed and written json metadata back into memory
+    root_metadata_path = os.path.join(
+        repo_dir, repo_tool.METADATA_STAGED_DIRECTORY_NAME, "root.json")
+    root_metadata = securesystemslib.util.load_json_file(root_metadata_path)
+
+    # Assert signatures are ordered alphabetically (by signing key keyid)
+    self.assertListEqual(
+        [sig["keyid"] for sig in root_metadata["signatures"]],
+        [
+          "59a4df8af818e9ed7abe0764c0b47b4240952aa0d179b5b78346c470ac30278d",
+          "65171251a9aff5a8b3143a813481cb07f6e0de4eb197c767837fe4491b739093"
+        ])
 
 
 
@@ -1161,9 +1181,6 @@ class TestSnapshot(unittest.TestCase):
 class TestTargets(unittest.TestCase):
   @classmethod
   def setUpClass(cls):
-
-    # setUpClass() is called before tests in an individual class are executed.
-
     # Create a temporary directory to store the repository, metadata, and target
     # files.  'temporary_directory' must be deleted in TearDownClass() so that
     # temporary files are always removed, even when exceptions occur.
@@ -1173,10 +1190,6 @@ class TestTargets(unittest.TestCase):
 
   @classmethod
   def tearDownClass(cls):
-
-    # tearDownModule() is called after all the tests have run.
-    # http://docs.python.org/2/library/unittest.html#class-and-module-fixtures
-
     # Remove the temporary repository directory, which should contain all the
     # metadata, targets, and key files generated for the test cases.
     shutil.rmtree(cls.temporary_directory)
@@ -1923,9 +1936,6 @@ class TestTargets(unittest.TestCase):
 class TestRepositoryToolFunctions(unittest.TestCase):
   @classmethod
   def setUpClass(cls):
-
-    # setUpClass() is called before tests in an individual class are executed.
-
     # Create a temporary directory to store the repository, metadata, and target
     # files.  'temporary_directory' must be deleted in TearDownClass() so that
     # temporary files are always removed, even when exceptions occur.
@@ -1935,10 +1945,6 @@ class TestRepositoryToolFunctions(unittest.TestCase):
 
   @classmethod
   def tearDownClass(cls):
-
-    # tearDownModule() is called after all the tests have run.
-    # http://docs.python.org/2/library/unittest.html#class-and-module-fixtures
-
     # Remove the temporary repository directory, which should contain all the
     # metadata, targets, and key files generated for the test cases.
     shutil.rmtree(cls.temporary_directory)
@@ -2197,4 +2203,5 @@ class TestRepositoryToolFunctions(unittest.TestCase):
 
 # Run the test cases.
 if __name__ == '__main__':
+  utils.configure_test_logging(sys.argv)
   unittest.main()
