@@ -29,8 +29,8 @@ import os
 import logging
 import tempfile
 import shutil
-import sys
 import unittest
+import sys
 
 import tuf
 import tuf.log
@@ -40,6 +40,8 @@ import tuf.roledb
 import tuf.keydb
 import tuf.repository_tool as repo_tool
 
+from tests import utils
+
 import securesystemslib
 import securesystemslib.storage
 
@@ -228,4 +230,5 @@ class TestRepository(unittest.TestCase):
 
 
 if __name__ == '__main__':
+  utils.configure_test_logging(sys.argv)
   unittest.main()
