@@ -55,11 +55,10 @@ import shutil
 import copy
 import tempfile
 import logging
-import random
-import subprocess
-import sys
 import errno
+import sys
 import unittest
+import json
 
 import tuf
 import tuf.exceptions
@@ -72,6 +71,8 @@ import tuf.repository_lib as repo_lib
 import tuf.unittest_toolbox as unittest_toolbox
 import tuf.client.updater as updater
 
+from tests import utils
+
 import securesystemslib
 import six
 
@@ -83,8 +84,6 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
 
   @classmethod
   def setUpClass(cls):
-    # setUpClass() is called before tests in an individual class are executed.
-
     # Create a temporary directory to store the repository, metadata, and target
     # files.  'temporary_directory' must be deleted in TearDownModule() so that
     # temporary files are always removed, even when exceptions occur.
@@ -102,39 +101,18 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     # assume the pre-generated metadata files have a specific structure, such
     # as a delegated role 'targets/role1', three target files, five key files,
     # etc.
-    cls.SERVER_PORT = random.randint(30000, 45000)
-    command = ['python', cls.SIMPLE_SERVER_PATH, str(cls.SERVER_PORT)]
-    cls.server_process = subprocess.Popen(command)
-    logger.info('\n\tServer process started.')
-    logger.info('\tServer process id: '+str(cls.server_process.pid))
-    logger.info('\tServing on port: '+str(cls.SERVER_PORT))
-    cls.url = 'http://localhost:'+str(cls.SERVER_PORT) + os.path.sep
-
-    # NOTE: Following error is raised if a delay is not long enough to allow
-    # the server process to set up and start listening:
-    #     <urlopen error [Errno 111] Connection refused>
-    # or, on Windows:
-    #     Failed to establish a new connection: [Errno 111] Connection refused'
-    # While 0.3s has consistently worked on Travis and local builds, it led to
-    # occasional failures in AppVeyor builds, so increasing this to 2s, sadly.
-    time.sleep(2)
+    cls.server_process_handler = utils.TestServerProcess(log=logger,
+        server=cls.SIMPLE_SERVER_PATH)
 
 
 
   @classmethod
   def tearDownClass(cls):
-    # tearDownModule() is called after all the tests have run.
-    # http://docs.python.org/2/library/unittest.html#class-and-module-fixtures
-
-    # Kill the SimpleHTTPServer process.
-    if cls.server_process.returncode is None:
-      logger.info('\tServer process ' + str(cls.server_process.pid) + ' terminated.')
-      cls.server_process.kill()
+    # Cleans the resources and flush the logged lines (if any).
+    cls.server_process_handler.clean()
 
     # Remove the temporary repository directory, which should contain all the
-    # metadata, targets, and key files generated for the test cases.  sleep
-    # for a bit to allow the kill'd server process to terminate.
-    time.sleep(.3)
+    # metadata, targets, and key files generated for the test cases
     shutil.rmtree(cls.temporary_directory)
 
 
@@ -184,8 +162,8 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
 
     # 'path/to/tmp/repository' -> 'localhost:8001/tmp/repository'.
     repository_basepath = self.repository_directory[len(os.getcwd()):]
-    url_prefix = \
-      'http://localhost:' + str(self.SERVER_PORT) + repository_basepath
+    url_prefix = 'http://localhost:' \
+        + str(self.server_process_handler.port) + repository_basepath
 
     # Setting 'tuf.settings.repository_directory' with the temporary client
     # directory copied from the original repository files.
@@ -193,8 +171,7 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
 
     self.repository_mirrors = {'mirror1': {'url_prefix': url_prefix,
                                            'metadata_path': 'metadata',
-                                           'targets_path': 'targets',
-                                           'confined_target_dirs': ['']}}
+                                           'targets_path': 'targets'}}
 
     # Creating a repository instance.  The test cases will use this client
     # updater to refresh metadata, fetch target files, etc.
@@ -214,7 +191,8 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     tuf.roledb.clear_roledb(clear_all=True)
     tuf.keydb.clear_keydb(clear_all=True)
 
-
+    # Logs stdout and stderr from the sever subprocess.
+    self.server_process_handler.flush_log()
 
 
   # UNIT TESTS.
@@ -246,6 +224,9 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     # directory.
     tuf.settings.repositories_directory = self.client_directory
 
+    # Test: repository does not exist
+    self.assertRaises(tuf.exceptions.MissingLocalRepositoryError, updater.Updater,
+                      'test_non_existing_repository', self.repository_mirrors)
 
     # Test: empty client repository (i.e., no metadata directory).
     metadata_backup = self.client_metadata + '.backup'
@@ -359,7 +340,7 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     # 'targets.json' are also loaded when the repository object is
     # instantiated.
 
-    self.assertEqual(number_of_root_keys * 2 + 2, len(tuf.keydb._keydb_dict[self.repository_name]))
+    self.assertEqual(number_of_root_keys + 1, len(tuf.keydb._keydb_dict[self.repository_name]))
 
     # Test: normal case.
     self.repository_updater._rebuild_key_and_role_db()
@@ -370,7 +351,7 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     # _rebuild_key_and_role_db() will only rebuild the keys and roles specified
     # in the 'root.json' file, unlike __init__().  Instantiating an updater
     # object calls both _rebuild_key_and_role_db() and _import_delegations().
-    self.assertEqual(number_of_root_keys * 2, len(tuf.keydb._keydb_dict[self.repository_name]))
+    self.assertEqual(number_of_root_keys, len(tuf.keydb._keydb_dict[self.repository_name]))
 
     # Test: properly updated roledb and keydb dicts if the Root role changes.
     root_metadata = self.repository_updater.metadata['current']['root']
@@ -381,7 +362,7 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
 
     root_roleinfo = tuf.roledb.get_roleinfo('root', self.repository_name)
     self.assertEqual(root_roleinfo['threshold'], 8)
-    self.assertEqual(number_of_root_keys * 2 - 2, len(tuf.keydb._keydb_dict[self.repository_name]))
+    self.assertEqual(number_of_root_keys - 1, len(tuf.keydb._keydb_dict[self.repository_name]))
 
 
 
@@ -567,7 +548,7 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
 
     # Take into account the number of keyids algorithms supported by default,
     # which this test condition expects to be two (sha256 and sha512).
-    self.assertEqual(4 * 2, len(tuf.keydb._keydb_dict[repository_name]))
+    self.assertEqual(4, len(tuf.keydb._keydb_dict[repository_name]))
 
     # Test: pass a role without delegations.
     self.repository_updater._import_delegations('root')
@@ -576,8 +557,8 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     # checking the number of elements in the dictionaries.
     self.assertEqual(len(tuf.roledb._roledb_dict[repository_name]), 4)
     # Take into account the number of keyid hash algorithms, which this
-    # test condition expects to be two (for sha256 and sha512).
-    self.assertEqual(len(tuf.keydb._keydb_dict[repository_name]), 4 * 2)
+    # test condition expects to be one
+    self.assertEqual(len(tuf.keydb._keydb_dict[repository_name]), 4)
 
     # Test: normal case, first level delegation.
     self.repository_updater._import_delegations('targets')
@@ -585,7 +566,7 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     self.assertEqual(len(tuf.roledb._roledb_dict[repository_name]), 5)
     # The number of root keys (times the number of key hash algorithms) +
     # delegation's key (+1 for its sha512 keyid).
-    self.assertEqual(len(tuf.keydb._keydb_dict[repository_name]), 4 * 2 + 2)
+    self.assertEqual(len(tuf.keydb._keydb_dict[repository_name]), 4 + 1)
 
     # Verify that roledb dictionary was added.
     self.assertTrue('role1' in tuf.roledb._roledb_dict[repository_name])
@@ -1022,7 +1003,8 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
    self.repository_updater.refresh()
 
    # Test: normal case.
-   all_targets = self.repository_updater.all_targets()
+   with utils.ignore_deprecation_warnings('tuf.client.updater'):
+    all_targets = self.repository_updater.all_targets()
 
    # Verify format of 'all_targets', it should correspond to
    # 'TARGETINFOS_SCHEMA'.
@@ -1066,7 +1048,8 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
 
 
     # Test: normal case.
-    targetinfos = self.repository_updater.targets_of_role('role1')
+    with utils.ignore_deprecation_warnings('tuf.client.updater'):
+      targetinfos = self.repository_updater.targets_of_role('role1')
 
     # Verify that the expected role files were downloaded and installed.
     os.path.exists(os.path.join(self.client_metadata_current, 'targets.json'))
@@ -1083,10 +1066,11 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
 
     # Test: Invalid arguments.
     # targets_of_role() expected a string rolename.
-    self.assertRaises(securesystemslib.exceptions.FormatError, self.repository_updater.targets_of_role,
-                      8)
-    self.assertRaises(tuf.exceptions.UnknownRoleError, self.repository_updater.targets_of_role,
-                      'unknown_rolename')
+    with utils.ignore_deprecation_warnings('tuf.client.updater'):
+      self.assertRaises(securesystemslib.exceptions.FormatError, self.repository_updater.targets_of_role,
+                        8)
+      self.assertRaises(tuf.exceptions.UnknownRoleError, self.repository_updater.targets_of_role,
+                        'unknown_rolename')
 
 
 
@@ -1097,26 +1081,18 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     # Unlike some of the other tests, start up a fresh server here.
     # The SimpleHTTPServer started in the setupclass has a tendency to
     # timeout in Windows after a few tests.
-    SERVER_PORT = random.randint(30000, 45000)
-    command = ['python', self.SIMPLE_SERVER_PATH, str(SERVER_PORT)]
-    server_process = subprocess.Popen(command)
 
-    # NOTE: Following error is raised if a delay is not long enough:
-    # <urlopen error [Errno 111] Connection refused>
-    # or, on Windows:
-    # Failed to establish a new connection: [Errno 111] Connection refused'
-    # While 0.3s has consistently worked on Travis and local builds, it led to
-    # occasional failures in AppVeyor builds, so increasing this to 2s, sadly.
-    time.sleep(2)
+    # Creates a subprocess running a server.
+    server_process_handler = utils.TestServerProcess(log=logger,
+        server=self.SIMPLE_SERVER_PATH)
 
     # 'path/to/tmp/repository' -> 'localhost:8001/tmp/repository'.
     repository_basepath = self.repository_directory[len(os.getcwd()):]
-    url_prefix = \
-      'http://localhost:' + str(SERVER_PORT) + repository_basepath
+    url_prefix = 'http://localhost:' \
+        + str(self.server_process_handler.port) + repository_basepath
 
     self.repository_mirrors = {'mirror1': {'url_prefix': url_prefix,
-        'metadata_path': 'metadata', 'targets_path': 'targets',
-        'confined_target_dirs': ['']}}
+        'metadata_path': 'metadata', 'targets_path': 'targets'}}
 
     # Creating a repository instance.  The test cases will use this client
     # updater to refresh metadata, fetch target files, etc.
@@ -1230,7 +1206,8 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
         self.repository_updater.get_one_valid_targetinfo,
         '/foo/foo1.1.tar.gz')
 
-    server_process.kill()
+    # Cleans the resources and flush the logged lines (if any).
+    server_process_handler.clean()
 
 
 
@@ -1303,6 +1280,39 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     self.repository_updater.download_target(targetinfo2,
                                             destination_directory)
 
+    # Checks if the file has been successfully downloaded
+    download_filepath = os.path.join(destination_directory, target_filepath2)
+    self.assertTrue(os.path.exists(download_filepath))
+
+    # Removes the file so that it can be downloaded again in the next test
+    os.remove(download_filepath)
+
+    # Test downloading with consistent snapshot enabled, but without adding
+    # the hash of the file as a prefix to its name.
+
+    file1_path = targetinfo2['filepath']
+    file1_hashes = securesystemslib.util.get_file_hashes(
+        os.path.join(self.repository_directory, 'targets', file1_path),
+        hash_algorithms=['sha256', 'sha512'])
+
+    # Currently in the repository directory, those three files exists:
+    # "file1.txt", "<sha256_hash>.file1.txt" and "<sha512_hash>.file1.txt"
+    # where both sha256 and sha512 hashes are for file file1.txt.
+    # Remove the files with the hash digest prefix to ensure that
+    # the served target file is not prefixed.
+    os.remove(os.path.join(self.repository_directory, 'targets',
+        file1_hashes['sha256'] + '.' + file1_path))
+    os.remove(os.path.join(self.repository_directory, 'targets',
+        file1_hashes['sha512'] + '.' + file1_path))
+
+
+    self.repository_updater.download_target(targetinfo2,
+                                            destination_directory,
+                                            prefix_filename_with_hash=False)
+
+    # Checks if the file has been successfully downloaded
+    self.assertTrue(os.path.exists(download_filepath))
+
     # Test for a destination that cannot be written to (apart from a target
     # file that already exists at the destination) and which raises an
     # exception.
@@ -1365,31 +1375,22 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     # Unlike some of the other tests, start up a fresh server here.
     # The SimpleHTTPServer started in the setupclass has a tendency to
     # timeout in Windows after a few tests.
-    SERVER_PORT = random.randint(30000, 45000)
-    command = ['python', self.SIMPLE_SERVER_PATH, str(SERVER_PORT)]
-    server_process = subprocess.Popen(command)
 
-    # NOTE: Following error is raised if a delay is not long enough to allow
-    # the server process to set up and start listening:
-    #     <urlopen error [Errno 111] Connection refused>
-    # or, on Windows:
-    #     Failed to establish a new connection: [Errno 111] Connection refused'
-    # While 0.3s has consistently worked on Travis and local builds, it led to
-    # occasional failures in AppVeyor builds, so increasing this to 2s, sadly.
-    time.sleep(2)
+    # Creates a subprocess running a server.
+    server_process_handler = utils.TestServerProcess(log=logger,
+        server=self.SIMPLE_SERVER_PATH)
 
     # 'path/to/tmp/repository' -> 'localhost:8001/tmp/repository'.
     repository_basepath = self.repository_directory[len(os.getcwd()):]
-    url_prefix = \
-      'http://localhost:' + str(SERVER_PORT) + repository_basepath
+    url_prefix = 'http://localhost:' \
+        + str(self.server_process_handler.port) + repository_basepath
 
     # Setting 'tuf.settings.repository_directory' with the temporary client
     # directory copied from the original repository files.
     tuf.settings.repositories_directory = self.client_directory
 
     self.repository_mirrors = {'mirror1': {'url_prefix': url_prefix,
-        'metadata_path': 'metadata', 'targets_path': 'targets',
-        'confined_target_dirs': ['']}}
+        'metadata_path': 'metadata', 'targets_path': 'targets'}}
 
     # Creating a repository instance.  The test cases will use this client
     # updater to refresh metadata, fetch target files, etc.
@@ -1401,7 +1402,8 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
 
     # Get the list of target files.  It will be used as an argument to the
     # 'updated_targets()' function.
-    all_targets = self.repository_updater.all_targets()
+    with utils.ignore_deprecation_warnings('tuf.client.updater'):
+      all_targets = self.repository_updater.all_targets()
 
     # Test for duplicates and targets in the root directory of the repository.
     additional_target = all_targets[0].copy()
@@ -1415,7 +1417,8 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     updated_targets = \
       self.repository_updater.updated_targets(all_targets, destination_directory)
 
-    all_targets = self.repository_updater.all_targets()
+    with utils.ignore_deprecation_warnings('tuf.client.updater'):
+      all_targets = self.repository_updater.all_targets()
 
     # Assumed the pre-generated repository specifies two target files in
     # 'targets.json' and one delegated target file in 'role1.json'.
@@ -1482,12 +1485,14 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     self.repository_updater.refresh()
 
     # Verify that the new target file is considered updated.
-    all_targets = self.repository_updater.all_targets()
+    with utils.ignore_deprecation_warnings('tuf.client.updater'):
+      all_targets = self.repository_updater.all_targets()
     updated_targets = \
       self.repository_updater.updated_targets(all_targets, destination_directory)
     self.assertEqual(len(updated_targets), 1)
 
-    server_process.kill()
+    # Cleans the resources and flush the logged lines (if any).
+    server_process_handler.clean()
 
 
 
@@ -1497,31 +1502,22 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     # Unlike some of the other tests, start up a fresh server here.
     # The SimpleHTTPServer started in the setupclass has a tendency to
     # timeout in Windows after a few tests.
-    SERVER_PORT = random.randint(30000, 45000)
-    command = ['python', self.SIMPLE_SERVER_PATH, str(SERVER_PORT)]
-    server_process = subprocess.Popen(command)
 
-    # NOTE: Following error is raised if a delay is not long enough to allow
-    # the server process to set up and start listening:
-    #     <urlopen error [Errno 111] Connection refused>
-    # or, on Windows:
-    #     Failed to establish a new connection: [Errno 111] Connection refused'
-    # While 0.3s has consistently worked on Travis and local builds, it led to
-    # occasional failures in AppVeyor builds, so increasing this to 2s, sadly.
-    time.sleep(2)
+    # Creates a subprocess running a server.
+    server_process_handler = utils.TestServerProcess(log=logger,
+        server=self.SIMPLE_SERVER_PATH)
 
     # 'path/to/tmp/repository' -> 'localhost:8001/tmp/repository'.
     repository_basepath = self.repository_directory[len(os.getcwd()):]
-    url_prefix = \
-      'http://localhost:' + str(SERVER_PORT) + repository_basepath
+    url_prefix = 'http://localhost:' \
+        + str(self.server_process_handler.port) + repository_basepath
 
     # Setting 'tuf.settings.repository_directory' with the temporary client
     # directory copied from the original repository files.
     tuf.settings.repositories_directory = self.client_directory
 
     self.repository_mirrors = {'mirror1': {'url_prefix': url_prefix,
-        'metadata_path': 'metadata', 'targets_path': 'targets',
-        'confined_target_dirs': ['']}}
+        'metadata_path': 'metadata', 'targets_path': 'targets'}}
 
     # Creating a repository instance.  The test cases will use this client
     # updater to refresh metadata, fetch target files, etc.
@@ -1532,7 +1528,8 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     destination_directory = self.make_temp_directory()
 
     #  Populate 'destination_direction' with all target files.
-    all_targets = self.repository_updater.all_targets()
+    with utils.ignore_deprecation_warnings('tuf.client.updater'):
+      all_targets = self.repository_updater.all_targets()
 
     self.assertEqual(len(os.listdir(destination_directory)), 0)
 
@@ -1563,7 +1560,8 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     # Verify number of target files in 'destination_directory' (should be 1
     # after the update made to the remote repository), and call
     # 'remove_obsolete_targets()'.
-    all_targets = self.repository_updater.all_targets()
+    with utils.ignore_deprecation_warnings('tuf.client.updater'):
+      all_targets = self.repository_updater.all_targets()
 
     updated_targets = \
       self.repository_updater.updated_targets(all_targets,
@@ -1596,7 +1594,8 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
     del self.repository_updater.metadata['previous']['targets']
     self.repository_updater.remove_obsolete_targets(destination_directory)
 
-    server_process.kill()
+    # Cleans the resources and flush the logged lines (if any).
+    server_process_handler.clean()
 
 
 
@@ -1618,38 +1617,19 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
 
 
 
-  def test_10__hard_check_file_length(self):
+  def test_10__check_file_length(self):
     # Test for exception if file object is not equal to trusted file length.
-    temp_file_object = tempfile.TemporaryFile()
-    temp_file_object.write(b'X')
-    temp_file_object.seek(0)
-    self.assertRaises(tuf.exceptions.DownloadLengthMismatchError,
-                     self.repository_updater._hard_check_file_length,
-                     temp_file_object, 10)
+    with tempfile.TemporaryFile() as temp_file_object:
+      temp_file_object.write(b'X')
+      temp_file_object.seek(0)
+      self.assertRaises(tuf.exceptions.DownloadLengthMismatchError,
+                      self.repository_updater._check_file_length,
+                      temp_file_object, 10)
 
 
 
 
 
-  def test_10__soft_check_file_length(self):
-    # Test for exception if file object is not equal to trusted file length.
-    temp_file_object = tempfile.TemporaryFile()
-    temp_file_object.write(b'XXX')
-    temp_file_object.seek(0)
-    self.assertRaises(tuf.exceptions.DownloadLengthMismatchError,
-                     self.repository_updater._soft_check_file_length,
-                     temp_file_object, 1)
-
-    # Verify that an exception is not raised if the file length <= the observed
-    # file length.
-    temp_file_object.seek(0)
-    self.repository_updater._soft_check_file_length(temp_file_object, 3)
-    temp_file_object.seek(0)
-    self.repository_updater._soft_check_file_length(temp_file_object, 4)
-
-
-
-
   def test_10__targets_of_role(self):
     # Test for non-existent role.
     self.assertRaises(tuf.exceptions.UnknownRoleError,
@@ -1754,58 +1734,18 @@ class TestUpdater(unittest_toolbox.Modified_TestCase):
 
 
 
-  def test_11__verify_uncompressed_metadata_file(self):
+  def test_11__verify_metadata_file(self):
     # Test for invalid metadata content.
-    metadata_file_object = tempfile.TemporaryFile()
-    metadata_file_object.write(b'X')
-    metadata_file_object.seek(0)
+    with tempfile.TemporaryFile() as metadata_file_object:
+      metadata_file_object.write(b'X')
+      metadata_file_object.seek(0)
 
-    self.assertRaises(tuf.exceptions.InvalidMetadataJSONError,
-        self.repository_updater._verify_uncompressed_metadata_file,
-        metadata_file_object, 'root')
+      self.assertRaises(tuf.exceptions.InvalidMetadataJSONError,
+          self.repository_updater._verify_metadata_file,
+          metadata_file_object, 'root')
 
 
-
-  def test_12__verify_root_chain_link(self):
-    # Test for an invalid signature in the chain link.
-    # current = (i.e., 1.root.json)
-    # next = signable for the next metadata in the chain (i.e., 2.root.json)
-    rolename = 'root'
-    current_root = self.repository_updater.metadata['current']['root']
-
-    targets_path = os.path.join(self.repository_directory, 'metadata', 'targets.json')
-
-    # 'next_invalid_root' is a Targets signable, as written to disk.
-    # We use the Targets metadata here to ensure the signatures are invalid.
-    next_invalid_root = securesystemslib.util.load_json_file(targets_path)
-
-    self.assertRaises(securesystemslib.exceptions.BadSignatureError,
-        self.repository_updater._verify_root_chain_link, rolename, current_root,
-        next_invalid_root)
-
-
-
-  def test_13__get_file(self):
-    # Test for an "unsafe" download, where the file is downloaded up to
-    # a required length (and no more).  The "safe" download approach
-    # downloads an exact required length.
-    targets_path = os.path.join(self.repository_directory, 'metadata', 'targets.json')
-
-    file_size, file_hashes = securesystemslib.util.get_file_details(targets_path)
-    file_type = 'meta'
-
-    def verify_target_file(targets_path):
-      # Every target file must have its length and hashes inspected.
-      self.repository_updater._hard_check_file_length(targets_path, file_size)
-      self.repository_updater._check_hashes(targets_path, file_hashes)
-
-    self.repository_updater._get_file('targets.json', verify_target_file,
-        file_type, file_size, download_safely=True)
-
-    self.repository_updater._get_file('targets.json', verify_target_file,
-        file_type, file_size, download_safely=False)
-
-  def test_14__targets_of_role(self):
+  def test_13__targets_of_role(self):
     # Test case where a list of targets is given.  By default, the 'targets'
     # parameter is None.
     targets = [{'filepath': 'file1.txt', 'fileinfo': {'length': 1, 'hashes': {'sha256': 'abc'}}}]
@@ -1884,47 +1824,42 @@ class TestMultiRepoUpdater(unittest_toolbox.Modified_TestCase):
     # the pre-generated metadata files have a specific structure, such
     # as a delegated role 'targets/role1', three target files, five key files,
     # etc.
-    self.SERVER_PORT = 30001
-    self.SERVER_PORT2 = 30002
 
-    command = ['python', self.SIMPLE_SERVER_PATH, str(self.SERVER_PORT)]
-    command2 = ['python', self.SIMPLE_SERVER_PATH, str(self.SERVER_PORT2)]
-
-    self.server_process = subprocess.Popen(command,
-        cwd=self.repository_directory)
+    # Creates a subprocess running a server.
+    self.server_process_handler = utils.TestServerProcess(log=logger,
+        server=self.SIMPLE_SERVER_PATH, popen_cwd=self.repository_directory)
 
     logger.debug('Server process started.')
-    logger.debug('Server process id: ' + str(self.server_process.pid))
-    logger.debug('Serving on port: ' + str(self.SERVER_PORT))
 
-    self.server_process2 = subprocess.Popen(command2,
-        cwd=self.repository_directory2)
+    # Creates a subprocess running a server.
+    self.server_process_handler2 = utils.TestServerProcess(log=logger,
+        server=self.SIMPLE_SERVER_PATH, popen_cwd=self.repository_directory2)
 
     logger.debug('Server process 2 started.')
-    logger.debug('Server 2 process id: ' + str(self.server_process2.pid))
-    logger.debug('Serving 2 on port: ' + str(self.SERVER_PORT2))
-    self.url = 'http://localhost:' + str(self.SERVER_PORT) + os.path.sep
-    self.url2 = 'http://localhost:' + str(self.SERVER_PORT2) + os.path.sep
 
-    # NOTE: Following error is raised if a delay is not long enough to allow
-    # the server process to set up and start listening:
-    #     <urlopen error [Errno 111] Connection refused>
-    # or, on Windows:
-    #     Failed to establish a new connection: [Errno 111] Connection refused'
-    # While 0.3s has consistently worked on Travis and local builds, it led to
-    # occasional failures in AppVeyor builds, so increasing this to 2s, sadly.
-    time.sleep(2)
+    url_prefix = 'http://localhost:' + str(self.server_process_handler.port)
+    url_prefix2 = 'http://localhost:' + str(self.server_process_handler2.port)
 
-    url_prefix = 'http://localhost:' + str(self.SERVER_PORT)
-    url_prefix2 = 'http://localhost:' + str(self.SERVER_PORT2)
+    # We have all of the necessary information for two repository mirrors
+    # in map.json, except for url prefixes.
+    # For the url prefixes, we create subprocesses that run a server script.
+    # In server scripts we get a free port from the OS which is sent
+    # back to the parent process.
+    # That's why we dynamically add the ports to the url prefixes
+    # and changing the content of map.json.
+    self.map_file_path = os.path.join(self.client_directory, 'map.json')
+    data = securesystemslib.util.load_json_file(self.map_file_path)
+
+    data['repositories']['test_repository1'] = [url_prefix]
+    data['repositories']['test_repository2'] = [url_prefix2]
+    with open(self.map_file_path, 'w') as f:
+      json.dump(data, f)
 
     self.repository_mirrors = {'mirror1': {'url_prefix': url_prefix,
-        'metadata_path': 'metadata', 'targets_path': 'targets',
-        'confined_target_dirs': ['']}}
+        'metadata_path': 'metadata', 'targets_path': 'targets'}}
 
     self.repository_mirrors2 = {'mirror1': {'url_prefix': url_prefix2,
-        'metadata_path': 'metadata', 'targets_path': 'targets',
-        'confined_target_dirs': ['']}}
+        'metadata_path': 'metadata', 'targets_path': 'targets'}}
 
     # Create the repository instances.  The test cases will use these client
     # updaters to refresh metadata, fetch target files, etc.
@@ -1949,23 +1884,16 @@ class TestMultiRepoUpdater(unittest_toolbox.Modified_TestCase):
     # directories that may have been created during each test case.
     unittest_toolbox.Modified_TestCase.tearDown(self)
 
-    # Kill the SimpleHTTPServer process.
-    if self.server_process.returncode is None:
-      logger.info('Server process ' + str(self.server_process.pid) + ' terminated.')
-      self.server_process.kill()
-
-    if self.server_process2.returncode is None:
-      logger.info('Server 2 process ' + str(self.server_process2.pid) + ' terminated.')
-      self.server_process2.kill()
+    # Cleans the resources and flush the logged lines (if any).
+    self.server_process_handler.clean()
+    self.server_process_handler2.clean()
 
     # updater.Updater() populates the roledb with the name "test_repository1"
     tuf.roledb.clear_roledb(clear_all=True)
     tuf.keydb.clear_keydb(clear_all=True)
 
     # Remove the temporary repository directory, which should contain all the
-    # metadata, targets, and key files generated of all the test cases.  sleep
-    # for a bit to allow the kill'd server processes to terminate.
-    time.sleep(.3)
+    # metadata, targets, and key files generated of all the test cases
     shutil.rmtree(self.temporary_directory)
 
 
@@ -1998,14 +1926,12 @@ class TestMultiRepoUpdater(unittest_toolbox.Modified_TestCase):
         updater.MultiRepoUpdater, root_filepath)
 
     # Test for a valid instantiation.
-    map_file = os.path.join(self.client_directory, 'map.json')
-    multi_repo_updater = updater.MultiRepoUpdater(map_file)
+    multi_repo_updater = updater.MultiRepoUpdater(self.map_file_path)
 
 
 
   def test__target_matches_path_pattern(self):
-    map_file = os.path.join(self.client_directory, 'map.json')
-    multi_repo_updater = updater.MultiRepoUpdater(map_file)
+    multi_repo_updater = updater.MultiRepoUpdater(self.map_file_path)
     paths = ['foo*.tgz', 'bar*.tgz', 'file1.txt']
     self.assertTrue(
         multi_repo_updater._target_matches_path_pattern('bar-1.0.tgz', paths))
@@ -2017,8 +1943,7 @@ class TestMultiRepoUpdater(unittest_toolbox.Modified_TestCase):
 
 
   def test_get_valid_targetinfo(self):
-    map_file = os.path.join(self.client_directory, 'map.json')
-    multi_repo_updater = updater.MultiRepoUpdater(map_file)
+    multi_repo_updater = updater.MultiRepoUpdater(self.map_file_path)
 
     # Verify the multi repo updater refuses to save targetinfo if
     # required local repositories are missing.
@@ -2125,8 +2050,7 @@ class TestMultiRepoUpdater(unittest_toolbox.Modified_TestCase):
 
 
   def test_get_updater(self):
-    map_file = os.path.join(self.client_directory, 'map.json')
-    multi_repo_updater = updater.MultiRepoUpdater(map_file)
+    multi_repo_updater = updater.MultiRepoUpdater(self.map_file_path)
 
     # Test for a non-existent repository name.
     self.assertEqual(None, multi_repo_updater.get_updater('bad_repo_name'))
@@ -2196,4 +2120,5 @@ def _load_role_keys(keystore_directory):
 
 
 if __name__ == '__main__':
+  utils.configure_test_logging(sys.argv)
   unittest.main()
