@@ -403,9 +403,9 @@ PROJECT_CFG_SCHEMA = SCHEMA.Object(
 MIRROR_SCHEMA = SCHEMA.Object(
   object_name = 'MIRROR_SCHEMA',
   url_prefix = securesystemslib.formats.URL_SCHEMA,
-  metadata_path = RELPATH_SCHEMA,
-  targets_path = RELPATH_SCHEMA,
-  confined_target_dirs = RELPATHS_SCHEMA,
+  metadata_path = SCHEMA.Optional(RELPATH_SCHEMA),
+  targets_path = SCHEMA.Optional(RELPATH_SCHEMA),
+  confined_target_dirs = SCHEMA.Optional(RELPATHS_SCHEMA),
   custom = SCHEMA.Optional(SCHEMA.Object()))
 
 # A dictionary of mirrors where the dict keys hold the mirror's name and
@@ -612,6 +612,37 @@ SCHEMAS_BY_TYPE = {
 
 
 
+
+def expiry_string_to_datetime(expires):
+  """
+  <Purpose>
+    Convert an expiry string to a datetime object.
+  <Arguments>
+    expires:
+      The expiry date-time string in the ISO8601 format that is defined
+      in securesystemslib.ISO8601_DATETIME_SCHEMA. E.g. '2038-01-19T03:14:08Z'
+  <Exceptions>
+    securesystemslib.exceptions.FormatError, if 'expires' cannot be
+    parsed correctly.
+  <Side Effects>
+    None.
+  <Returns>
+    A datetime object representing the expiry time.
+  """
+
+  # Raise 'securesystemslib.exceptions.FormatError' if there is a mismatch.
+  securesystemslib.formats.ISO8601_DATETIME_SCHEMA.check_match(expires)
+
+  try:
+    return datetime.datetime.strptime(expires, "%Y-%m-%dT%H:%M:%SZ")
+  except ValueError as error:
+    six.raise_from(securesystemslib.exceptions.FormatError(
+        'Failed to parse ' + repr(expires) + ' as an expiry time'),
+        error)
+
+
+
+
 def datetime_to_unix_timestamp(datetime_object):
   """
   <Purpose>
@@ -938,6 +969,9 @@ def check_signable_object_format(signable):
     securesystemslib.exceptions.FormatError, if 'signable' does not have the
     correct format.
 
+    tuf.exceptions.UnsignedMetadataError, if 'signable' does not have any
+    signatures
+
   <Side Effects>
     None.
 
@@ -954,15 +988,20 @@ def check_signable_object_format(signable):
   try:
     role_type = signable['signed']['_type']
 
-  except (KeyError, TypeError):
-    raise securesystemslib.exceptions.FormatError('Untyped signable object.')
+  except (KeyError, TypeError) as error:
+    six.raise_from(securesystemslib.exceptions.FormatError(
+        'Untyped signable object.'), error)
 
   try:
     schema = SCHEMAS_BY_TYPE[role_type]
 
-  except KeyError:
-    raise securesystemslib.exceptions.FormatError('Unrecognized'
-      ' type ' + repr(role_type))
+  except KeyError as error:
+    six.raise_from(securesystemslib.exceptions.FormatError(
+        'Unrecognized type ' + repr(role_type)), error)
+
+  if not signable['signatures']:
+    raise tuf.exceptions.UnsignedMetadataError('Signable object of type ' +
+        repr(role_type) + ' has no signatures ', signable)
 
   # 'securesystemslib.exceptions.FormatError' raised if 'signable' does not
   # have a properly formatted role schema.
