@@ -52,7 +52,6 @@ import securesystemslib
 import securesystemslib.hash
 import securesystemslib.interface
 import securesystemslib.util
-import iso8601
 import six
 
 import securesystemslib.storage
@@ -61,11 +60,6 @@ import securesystemslib.storage
 # See 'log.py' to learn how logging is handled in TUF.
 logger = logging.getLogger(__name__)
 
-# Disable 'iso8601' logger messages to prevent 'iso8601' from clogging the
-# log file.
-iso8601_logger = logging.getLogger('iso8601')
-iso8601_logger.disabled = True
-
 # The extension of TUF metadata.
 METADATA_EXTENSION = '.json'
 
@@ -118,7 +112,7 @@ def _generate_and_write_metadata(rolename, metadata_filename,
   roleinfo = tuf.roledb.get_roleinfo(rolename, repository_name)
   previous_keyids = roleinfo.get('previous_keyids', [])
   previous_threshold = roleinfo.get('previous_threshold', 1)
-  signing_keyids = list(set(roleinfo['signing_keyids']))
+  signing_keyids = sorted(set(roleinfo['signing_keyids']))
 
   # Generate the appropriate role metadata for 'rolename'.
   if rolename == 'root':
@@ -166,7 +160,13 @@ def _generate_and_write_metadata(rolename, metadata_filename,
     metadata = generate_targets_metadata(targets_directory,
         roleinfo['paths'], roleinfo['version'], roleinfo['expires'],
         roleinfo['delegations'], consistent_targets, use_existing_fileinfo,
-        storage_backend)
+        storage_backend, repository_name)
+
+    # Update roledb with the latest delegations info collected during
+    # generate_targets_metadata()
+    tuf.roledb.update_roleinfo(rolename, roleinfo,
+        repository_name=repository_name)
+
 
   # Before writing 'rolename' to disk, automatically increment its version
   # number (if 'increment_version_number' is True) so that the caller does not
@@ -499,7 +499,13 @@ def _load_top_level_metadata(repository, top_level_filenames, repository_name):
   try:
     # Initialize the key and role metadata of the top-level roles.
     signable = securesystemslib.util.load_json_file(root_filename)
-    tuf.formats.check_signable_object_format(signable)
+    try:
+      tuf.formats.check_signable_object_format(signable)
+    except tuf.exceptions.UnsignedMetadataError:
+      # Downgrade the error to a warning because a use case exists where
+      # metadata may be generated unsigned on one machine and signed on another.
+      logger.warning('Unsigned metadata object: ' + repr(signable))
+
     root_metadata = signable['signed']
     tuf.keydb.create_keydb_from_root_metadata(root_metadata, repository_name)
     tuf.roledb.create_roledb_from_root_metadata(root_metadata, repository_name)
@@ -534,9 +540,9 @@ def _load_top_level_metadata(repository, top_level_filenames, repository_name):
     # Ensure the 'consistent_snapshot' field is extracted.
     consistent_snapshot = root_metadata['consistent_snapshot']
 
-  except securesystemslib.exceptions.StorageError:
-    raise tuf.exceptions.RepositoryError('Cannot load the required'
-        ' root file: ' + repr(root_filename))
+  except securesystemslib.exceptions.StorageError as error:
+    six.raise_from(tuf.exceptions.RepositoryError('Cannot load the required'
+        ' root file: ' + repr(root_filename)), error)
 
   # Load 'timestamp.json'.  A Timestamp role file without a version number is
   # always written.
@@ -563,9 +569,9 @@ def _load_top_level_metadata(repository, top_level_filenames, repository_name):
     tuf.roledb.update_roleinfo('timestamp', roleinfo, mark_role_as_dirty=False,
         repository_name=repository_name)
 
-  except securesystemslib.exceptions.StorageError:
-    raise tuf.exceptions.RepositoryError('Cannot load the Timestamp file: '
-        + repr(timestamp_filename))
+  except securesystemslib.exceptions.StorageError as error:
+    six.raise_from(tuf.exceptions.RepositoryError('Cannot load the Timestamp '
+        'file: ' + repr(timestamp_filename)), error)
 
   # Load 'snapshot.json'.  A consistent snapshot.json must be calculated if
   # 'consistent_snapshot' is True.
@@ -580,7 +586,13 @@ def _load_top_level_metadata(repository, top_level_filenames, repository_name):
 
   try:
     signable = securesystemslib.util.load_json_file(snapshot_filename)
-    tuf.formats.check_signable_object_format(signable)
+    try:
+      tuf.formats.check_signable_object_format(signable)
+    except tuf.exceptions.UnsignedMetadataError:
+      # Downgrade the error to a warning because a use case exists where
+      # metadata may be generated unsigned on one machine and signed on another.
+      logger.warning('Unsigned metadata object: ' + repr(signable))
+
     snapshot_metadata = signable['signed']
 
     for signature in signable['signatures']:
@@ -603,9 +615,9 @@ def _load_top_level_metadata(repository, top_level_filenames, repository_name):
     tuf.roledb.update_roleinfo('snapshot', roleinfo, mark_role_as_dirty=False,
         repository_name=repository_name)
 
-  except securesystemslib.exceptions.StorageError:
-    raise tuf.exceptions.RepositoryError('The Snapshot file cannot be loaded: '
-        + repr(snapshot_filename))
+  except securesystemslib.exceptions.StorageError as error:
+    six.raise_from(tuf.exceptions.RepositoryError('The Snapshot file '
+        'cannot be loaded: '+ repr(snapshot_filename)), error)
 
   # Load 'targets.json'.  A consistent snapshot of the Targets role must be
   # calculated if 'consistent_snapshot' is True.
@@ -616,7 +628,13 @@ def _load_top_level_metadata(repository, top_level_filenames, repository_name):
 
   try:
     signable = securesystemslib.util.load_json_file(targets_filename)
-    tuf.formats.check_signable_object_format(signable)
+    try:
+      tuf.formats.check_signable_object_format(signable)
+    except tuf.exceptions.UnsignedMetadataError:
+      # Downgrade the error to a warning because a use case exists where
+      # metadata may be generated unsigned on one machine and signed on another.
+      logger.warning('Unsigned metadata object: ' + repr(signable))
+
     targets_metadata = signable['signed']
 
     for signature in signable['signatures']:
@@ -642,15 +660,11 @@ def _load_top_level_metadata(repository, top_level_filenames, repository_name):
         repository_name=repository_name)
 
     # Add the keys specified in the delegations field of the Targets role.
-    for key_metadata in six.itervalues(targets_metadata['delegations']['keys']):
+    for keyid, key_metadata in six.iteritems(targets_metadata['delegations']['keys']):
 
-      # The repo may have used hashing algorithms for the generated keyids
-      # that doesn't match the client's set of hash algorithms.  Make sure
-      # to only used the repo's selected hashing algorithms.
-      hash_algorithms = securesystemslib.settings.HASH_ALGORITHMS
-      securesystemslib.settings.HASH_ALGORITHMS = key_metadata['keyid_hash_algorithms']
-      key_object, keyids = securesystemslib.keys.format_metadata_to_key(key_metadata)
-      securesystemslib.settings.HASH_ALGORITHMS = hash_algorithms
+      # Use the keyid found in the delegation
+      key_object, _ = securesystemslib.keys.format_metadata_to_key(key_metadata,
+          keyid)
 
       # Add 'key_object' to the list of recognized keys.  Keys may be shared,
       # so do not raise an exception if 'key_object' has already been loaded.
@@ -659,17 +673,14 @@ def _load_top_level_metadata(repository, top_level_filenames, repository_name):
       # repository maintainer should have also been made aware of the duplicate
       # key when it was added.
       try:
-        for keyid in keyids: #pragma: no branch
-          key_object['keyid'] = keyid
-          tuf.keydb.add_key(key_object, keyid=None,
-              repository_name=repository_name)
+        tuf.keydb.add_key(key_object, keyid=None, repository_name=repository_name)
 
       except tuf.exceptions.KeyAlreadyExistsError:
         pass
 
-  except securesystemslib.exceptions.StorageError:
-    raise tuf.exceptions.RepositoryError('The Targets file can not be loaded: '
-        + repr(targets_filename))
+  except securesystemslib.exceptions.StorageError as error:
+    six.raise_from(tuf.exceptions.RepositoryError('The Targets file '
+        'can not be loaded: ' + repr(targets_filename)), error)
 
   return repository, consistent_snapshot
 
@@ -687,7 +698,8 @@ def _log_warning_if_expires_soon(rolename, expires_iso8601_timestamp,
   # unix timestamp, subtract from current time.time() (also in POSIX time)
   # and compare against 'seconds_remaining_to_warn'.  Log a warning message
   # to console if 'rolename' expires soon.
-  datetime_object = iso8601.parse_date(expires_iso8601_timestamp)
+  datetime_object = tuf.formats.expiry_string_to_datetime(
+      expires_iso8601_timestamp)
   expires_unix_timestamp = \
     tuf.formats.datetime_to_unix_timestamp(datetime_object)
   seconds_until_expires = expires_unix_timestamp - int(time.time())
@@ -1235,6 +1247,7 @@ def generate_root_metadata(version, expiration_date, consistent_snapshot,
   # Conformant to 'ROLEDICT_SCHEMA' and 'KEYDICT_SCHEMA', respectively.
   roledict = {}
   keydict = {}
+  keylist = []
 
   # Extract the role, threshold, and keyid information of the top-level roles,
   # which Root stores in its metadata.  The necessary role metadata is generated
@@ -1246,43 +1259,11 @@ def generate_root_metadata(version, expiration_date, consistent_snapshot,
       raise securesystemslib.exceptions.Error(repr(rolename) + ' not in'
           ' "tuf.roledb".')
 
-    # Keep track of the keys loaded to avoid duplicates.
-    keyids = []
-
-    # Generate keys for the keyids listed by the role being processed.
-    for keyid in tuf.roledb.get_role_keyids(rolename, repository_name):
+    # Collect keys from all roles in a list
+    keyids = tuf.roledb.get_role_keyids(rolename, repository_name)
+    for keyid in keyids:
       key = tuf.keydb.get_key(keyid, repository_name=repository_name)
-
-      # If 'key' is an RSA key, it would conform to
-      # 'securesystemslib.formats.RSAKEY_SCHEMA', and have the form:
-      # {'keytype': 'rsa',
-      #  'keyid': keyid,
-      #  'keyval': {'public': '-----BEGIN RSA PUBLIC KEY----- ...',
-      #             'private': '-----BEGIN RSA PRIVATE KEY----- ...'}}
-      keyid = key['keyid']
-      if keyid not in keydict:
-
-        # This appears to be a new keyid.  Generate the key for it.
-        if key['keytype'] in ['rsa', 'ed25519', 'ecdsa-sha2-nistp256']:
-          keytype = key['keytype']
-          keyval = key['keyval']
-          scheme = key['scheme']
-          keydict[keyid] = \
-            securesystemslib.keys.format_keyval_to_metadata(keytype,
-                scheme, keyval, private=False)
-
-        # This is not a recognized key.  Raise an exception.
-        else:
-          raise securesystemslib.exceptions.Error('Unsupported keytype:'
-          ' ' + key['keytype'])
-
-      # Do we have a duplicate?
-      if keyid in keyids:
-        raise securesystemslib.exceptions.Error('Same keyid listed twice:'
-          ' ' + keyid)
-
-      # Add the loaded keyid for the role being processed.
-      keyids.append(keyid)
+      keylist.append(key)
 
     # Generate the authentication information Root establishes for each
     # top-level role.
@@ -1293,6 +1274,9 @@ def generate_root_metadata(version, expiration_date, consistent_snapshot,
         threshold=role_threshold)
     roledict[rolename] = role_metadata
 
+  # Create the root metadata 'keys' dictionary
+  _, keydict = keys_to_keydict(keylist)
+
   # Use generalized build_dict_conforming_to_schema func to produce a dict that
   # contains all the appropriate information for this type of metadata,
   # checking that the result conforms to the appropriate schema.
@@ -1315,7 +1299,8 @@ def generate_root_metadata(version, expiration_date, consistent_snapshot,
 
 def generate_targets_metadata(targets_directory, target_files, version,
     expiration_date, delegations=None, write_consistent_targets=False,
-    use_existing_fileinfo=False, storage_backend=None):
+    use_existing_fileinfo=False, storage_backend=None,
+    repository_name='default'):
   """
   <Purpose>
     Generate the targets metadata object. The targets in 'target_files' must
@@ -1368,6 +1353,10 @@ def generate_targets_metadata(targets_directory, target_files, version,
       An object which implements
       securesystemslib.storage.StorageBackendInterface.
 
+    repository_name:
+      The name of the repository.  If not supplied, 'default' repository
+      is used.
+
   <Exceptions>
     securesystemslib.exceptions.FormatError, if an error occurred trying to
     generate the targets metadata object.
@@ -1412,6 +1401,23 @@ def generate_targets_metadata(targets_directory, target_files, version,
 
   if delegations is not None:
     tuf.formats.DELEGATIONS_SCHEMA.check_match(delegations)
+    # If targets role has delegations, collect the up-to-date 'keyids' and
+    # 'threshold' for each role. Update the delegations keys dictionary.
+    delegations_keys = []
+    # Update 'keyids' and 'threshold' for each delegated role
+    for role in delegations['roles']:
+      role['keyids'] = tuf.roledb.get_role_keyids(role['name'],
+          repository_name)
+      role['threshold'] = tuf.roledb.get_role_threshold(role['name'],
+          repository_name)
+
+      # Collect all delegations keys for generating the delegations keydict
+      for keyid in role['keyids']:
+        key = tuf.keydb.get_key(keyid, repository_name=repository_name)
+        delegations_keys.append(key)
+
+    _, delegations['keys'] = keys_to_keydict(delegations_keys)
+
 
   # Store the file attributes of targets in 'target_files'.  'filedict',
   # conformant to 'tuf.formats.FILEDICT_SCHEMA', is added to the
@@ -1425,13 +1431,13 @@ def generate_targets_metadata(targets_directory, target_files, version,
 
       # Ensure all fileinfo entries in target_files have a non-empty hashes dict
       if not fileinfo.get('hashes', None):
-        raise securesystemslib.exceptions.Error('use_existing_hashes option set'
-            ' but no hashes exist in roledb for ' + repr(target))
+        raise securesystemslib.exceptions.Error('use_existing_fileinfo option'
+            ' set but no hashes exist in fileinfo for ' + repr(target))
 
       # and a non-empty length
       if fileinfo.get('length', -1) < 0:
-        raise securesystemslib.exceptions.Error('use_existing_hashes option set'
-            ' but fileinfo\'s length is not set')
+        raise securesystemslib.exceptions.Error('use_existing_fileinfo option'
+            ' set but no length exists in fileinfo for ' + repr(target))
 
       filedict[target] = fileinfo
 
@@ -1521,6 +1527,28 @@ def _generate_targets_fileinfo(target_files, targets_directory,
 
 
 
+def _get_hashes_and_length_if_needed(use_length, use_hashes, full_file_path,
+    storage_backend):
+  """
+  Calculate length and hashes only if they are required,
+  otherwise, for adopters of tuf with lots of delegations,
+  this will cause unnecessary overhead.
+  """
+
+  length = None
+  hashes = None
+  if use_length:
+    length = securesystemslib.util.get_file_length(full_file_path,
+        storage_backend)
+
+  if use_hashes:
+    hashes = securesystemslib.util.get_file_hashes(full_file_path,
+        tuf.settings.FILE_HASH_ALGORITHMS, storage_backend)
+
+  return length, hashes
+
+
+
 def generate_snapshot_metadata(metadata_directory, version, expiration_date,
     storage_backend, consistent_snapshot=False,
     repository_name='default', use_length=False, use_hashes=False):
@@ -1604,12 +1632,8 @@ def generate_snapshot_metadata(metadata_directory, version, expiration_date,
   # Targets, and all delegated roles of the repository.
   fileinfodict = {}
 
-  length, hashes = securesystemslib.util.get_file_details(
-      os.path.join(metadata_directory, TARGETS_FILENAME),
-      tuf.settings.FILE_HASH_ALGORITHMS, storage_backend)
-
-  length = (use_length and length) or None
-  hashes = (use_hashes and hashes) or None
+  length, hashes = _get_hashes_and_length_if_needed(use_length, use_hashes,
+      os.path.join(metadata_directory, TARGETS_FILENAME), storage_backend)
 
   targets_role = TARGETS_FILENAME[:-len(METADATA_EXTENSION)]
 
@@ -1645,18 +1669,8 @@ def generate_snapshot_metadata(metadata_directory, version, expiration_date,
       if tuf.roledb.role_exists(rolename, repository_name) and \
           rolename not in tuf.roledb.TOP_LEVEL_ROLES:
 
-        length = None
-        hashes = None
-        # We want to make sure we are calculating length and hashes only when
-        # at least one of them is needed. Otherwise, for adoptors of tuf with
-        # lots of delegations, this will cause unnecessary overhead.
-        if use_length or use_hashes:
-          length, hashes = securesystemslib.util.get_file_details(
-              os.path.join(metadata_directory, metadata_filename),
-              tuf.settings.FILE_HASH_ALGORITHMS)
-
-          length = (use_length and length) or None
-          hashes = (use_hashes and hashes) or None
+        length, hashes = _get_hashes_and_length_if_needed(use_length, use_hashes,
+            os.path.join(metadata_directory, metadata_filename), storage_backend)
 
         file_version = get_metadata_versioninfo(rolename,
             repository_name)
@@ -1752,11 +1766,8 @@ def generate_timestamp_metadata(snapshot_file_path, version, expiration_date,
 
   snapshot_fileinfo = {}
 
-  length, hashes = securesystemslib.util.get_file_details(snapshot_file_path,
-      tuf.settings.FILE_HASH_ALGORITHMS, storage_backend)
-
-  length = (use_length and length) or None
-  hashes = (use_hashes and hashes) or None
+  length, hashes = _get_hashes_and_length_if_needed(use_length, use_hashes,
+      snapshot_file_path, storage_backend)
 
   snapshot_filename = os.path.basename(snapshot_file_path)
   # Retrieve the versioninfo of the Snapshot metadata file.
@@ -1864,7 +1875,13 @@ def sign_metadata(metadata_object, keyids, filename, repository_name):
 
   # Raise 'securesystemslib.exceptions.FormatError' if the resulting 'signable'
   # is not formatted correctly.
-  tuf.formats.check_signable_object_format(signable)
+  try:
+    tuf.formats.check_signable_object_format(signable)
+  except tuf.exceptions.UnsignedMetadataError:
+    # Downgrade the error to a warning because a use case exists where
+    # metadata may be generated unsigned on one machine and signed on another.
+    logger.warning('Unsigned metadata object: ' + repr(signable))
+
 
   return signable
 
@@ -2256,9 +2273,42 @@ def disable_console_log_messages():
 
 
 
+def keys_to_keydict(keys):
+  """
+  <Purpose>
+    Iterate over a list of keys and return a list of keyids and a dict mapping
+    keyid to key metadata
+
+  <Arguments>
+    keys:
+      A list of key objects conforming to
+      securesystemslib.formats.ANYKEYLIST_SCHEMA.
+
+  <Returns>
+    keyids:
+      A list of keyids conforming to securesystemslib.formats.KEYID_SCHEMA
+    keydict:
+      A dictionary conforming to securesystemslib.formats.KEYDICT_SCHEMA
+  """
+  keyids = []
+  keydict = {}
+
+  for key in keys:
+    keyid = key['keyid']
+    key_metadata_format = securesystemslib.keys.format_keyval_to_metadata(
+        key['keytype'], key['scheme'], key['keyval'])
+
+    new_keydict = {keyid: key_metadata_format}
+    keydict.update(new_keydict)
+    keyids.append(keyid)
+  return keyids, keydict
+
+
+
+
 if __name__ == '__main__':
   # The interactive sessions of the documentation strings can
-  # be tested by running repository_tool.py as a standalone module:
+  # be tested by running repository_lib.py as a standalone module:
   # $ python repository_lib.py.
   import doctest
   doctest.testmod()
